----------------------------
-- yoke © nico@yokerp.com --
----------------------------

local db = {}
local concat,insert = table.concat,table.insert

local sql = {
  {'sql',"SQL",ht = "TEXTAREA"},
  {"filename","File",sz = 40, ht = 'FILE',
      opt=[[onchange='fcpy(this,$("#sql"),"sql")']]},
  submit = {{"bases","Run",{act='sql'}}},
}

db.view = {
  {'user','User',ht="CHECKBOX",id='sys'},
  {'page','Page',ht="CHECKBOX",id='sys'},
  {'base','Base',ht="CHECKBOX",id='sys'},
  mod = "bases",
  submit = {
    {"bases","Migrate",{act='create'}},
  },
  columns = 2,
}

function db:main()
  if not self.user.admin then return end
  local act = params.act
  if not act then
    params.act = 'new'
    local rf = db.view
    if self.conf.cli then
      rf.submit[2] = {"bases","Drop",{act='drop'}} 
      rf.submit[3] = {"bases","Sql",{act='sql'}}
    end
    for i,app in ipairs(self.conf.apps or {}) do
      if app[2] then rf[i+2] = {app[1],app[2],ht="CHECKBOX",id='app'} end
    end
  elseif act == 'sql' and not params.sql then
    db.view = sql
    params.act = "new"
  elseif params.sql then
    local result = {}
    local function rows(udata,cols,values,names)
      local row = {}
      if #result == 0 then
        for i=1,cols do insert(row,names[i]) end
        insert(result,concat(row,","))
        row = {}
      end
      for i=1,cols do insert(row,values[i]) end
      insert(result,concat(row,","))
      return #result < 1000 and 0 or 1
    end
    local sql = params.sql
    local res = self.conn:exec(sql,rows,"rows")
    return concat{sql,"\n",res == 0 and concat(result,"\n") or
          self.conn:error_message()}
  elseif act == 'create' or act == 'drop' then
    local init = setmetatable(require"yoke.snap.init",{__index=self})
    local res = {}
    self.conn:trace(function(ud, sql) res[#res+1] = sql end)
    self.conn:exec("begin")
    local sys = type(params.sys) == 'table' and params.sys or {params.sys}
    local apps = type(params.app) == 'table' and params.app or {params.app}
    for _,aid in ipairs(sys) do
      if aid ~= '' then
        local conf = require(concat{'yoke.site.apps.',aid,'.conf'}) or {}
        for _,row in ipairs(conf) do
          local id = row[1]
          local mod = require(concat{"yoke.site.apps.",aid,".mods.",id})
          local model = id
          if not self.defs[model] then
            self.defs[model] = self:subclass(model):include(mod)
          end

          if type(mod)=='table' and mod.view and not mod.view.mod then
            if act == 'drop' then
              self.conn:exec(concat{'drop table ',id})
            elseif act == 'create' then
              --setmetatable(mod,{__index=self}) --todo check class
              if mod.field and mod.field[1] then mod.field[1](mod) end -- apps fields

              local ext = conf.extend and conf.extend[id] or nil
              if ext then
                --if ext.field and ext.field[1] then ext.field[1](mod) end
                if ext.field and ext.field[1] then ext.field[1](self.defs[model]) end --todo check class
              end

              local ok,mes = init:create(id,mod.view)
              if not ok then return mes end
            end
          end
        end
        if act == 'create' then --migrate
          local ok,def = pcall(require,concat{"yoke.site.apps.",aid,".meta.tail"})
          if not ok and self.conf.debug then
            res[#res+1] = def
          elseif type(def) == 'table' and def[1] then
            init:migrate(def,aid)
          end
        end
      end
    end

    for _,aid in ipairs(apps) do
      if aid ~= '' then
        local conf = self.conf.apps[aid] --require(concat{'apps.',aid,'.conf'}) or {}
        for _,row in ipairs(conf) do
          local id = row[1]

          local mod = require(concat{"apps.",aid,".mods.",id}) --todo object
          local model = concat{aid,'.',id}
          if not self.defs[model] then
            self.defs[model] = self:subclass(model):include(mod)
          end
          for a,app in ipairs(self.conf.apps or {}) do
            local config = self.conf.apps[app[1]] or {}
            for mid,ext in pairs(config.extend or {}) do
              if mid == id then
                 debug.log('EXTEND ID from APP',mid,app[1])
                 self.util.extend(self.defs[model],mod,ext)
              end
            end
          end
          if type(mod)=='table' and mod.view and not mod.view.mod then
            if act == 'drop' then
              self.conn:exec(concat{'drop table ',id})
            elseif act == 'create' then
              --setmetatable(mod,{__index=self}) --todo
              --if mod.field and type(mod.field[1]) == 'function' then mod.field[1](mod) end -- apps fields
              --local ext = conf.extend and conf.extend[id] or nil
              --if ext and ext.field and type(ext.field[1]) == 'function' then ext.field[1](mod) end

              local ok,mes = init:create(id,mod.view,aid)
              if not ok then return mes end
            end
          end
        end
        if act == 'create' then --migrate
          local ok,def = pcall(require,concat{"apps.",aid,".meta.tail"})
          if not ok and (self.conf.debug or self.conf.log) then
            res[#res+1] = def
          elseif type(def) == 'table' and def[1] then
            debug.log('MIGRATE',aid)
            init:migrate(def,aid)
          end
        end
      end
    end
    self.conn:exec("commit")
    self.conn:trace(nil)
    if self.conf.cli then return concat(res,"\n") end
    return {concat(res,"\r\n")}
  end
end

return db
