return {
camera = [[

<style>
#vid-container {
  position: fixed;
  top: 0;
  left: 0;
  text-align:center;
}

#video {
  width: 100%;
  height: 100%;
  object-fit: cover;
  z-index: 0;
}

#gui-controls {
  position: fixed;
  z-index: 2;
  bottom: 0;
}

#video-overlay {
  position: fixed;
  width: 100%;
  height: 100%;
  top: 0;
  left: 0;
  right: 0;
  bottom: 0;
  z-index: 10;
  text-align:center;
  margin-top: 10%;
}

button {
  outline: none;
  position: absolute;
  color: white;
  display: block;
  opacity: 1;
  background: transparent;
  border: solid 2px #fff;
  padding: 0;
  text-shadow: 0px 0px 4px black;
  background-position: center center;
  background-repeat: no-repeat;
  pointer-events: auto;
  z-index: 2;
}

#takePhotoButton {
  left: calc(50% - 40px);
  top: calc(50% - 40px);
  width: 80px;
  height: 80px;
  background-image: url('yoke/site/face/img/cam/photo.svg');
  border-radius: 50%;
  background-color: rgba(0, 0, 0, 0.5);
}

#takePhotoButton:active {
  background-color: #fff;
}

#toggleFullScreenButton {
  display: none;
  width: 64px;
  height: 64px;
  background-image: url('yoke/site/face/img/cam/full.svg');
  border-radius: 50%;
  background-color: rgba(0, 0, 0, 0.5);
}

#toggleFullScreenButton[aria-pressed='true'] {
  background-image: url('yoke/site/face/img/cam/fullexit.svg');
}

#switchCameraButton {
  display: none;
  width: 64px;
  height: 64px;
  background-image: url('yoke/site/face/img/cam/rear.svg');
  border-radius: 50%;
  background-color: rgba(0, 0, 0, 0.5);
}

#switchCameraButton[aria-pressed='true'] {
  background-image: url('yoke/site/face/img/cam/front.svg');
}

@media screen and (orientation: portrait) {
  #vid-container {
    width: 100%;
    height: 100%;
  }

  #video-overlay {
    width: 100%;
    height: 80%;
  }

  #gui-controls {
    width: 100%;
    height: 20%;
    left: 0;
  }

  #switchCameraButton {
    left: calc(20% - 32px);
    top: calc(50% - 32px);
  }

  #toggleFullScreenButton {
    left: calc(80% - 32px);
    top: calc(50% - 32px);
  }
}

@media screen and (orientation: landscape) {
  #vid_container {
    width: 100%;
    height: 100%;
  }

  #video-overlay {
    width: 100%;
    height: 100%;
    max-width:100%;
  }

  #vid-container.left {
    left: 20%;
  }

  #gui-controls {
    width: 20%;
    height: 100%;
    right: 0;
  }

  #gui-controls.left {
    left: 0;
  }

  #switchCameraButton {
    left: calc(50% - 32px);
    top: calc(18% - 32px);
  }

  #toggleFullScreenButton {
    left: calc(50% - 32px);
    top: calc(82% - 32px);
  }
}


</style>

<script type="text/javascript">

var takeSnapshotUI = createClickFeedbackUI();
var video;
var takePhotoButton;
var toggleFullScreenButton;
var switchCameraButton;
var amountOfCameras = 0;
var currentFacingMode = 'environment';

function deviceCount() {
  return new Promise(function (resolve) {
    var videoInCount = 0;

    navigator.mediaDevices
      .enumerateDevices()
      .then(function (devices) {
        devices.forEach(function (device) {
          if (device.kind === 'video') {
            device.kind = 'videoinput';
          }

          if (device.kind === 'videoinput') {
            videoInCount++;
          }
        });

        resolve(videoInCount);
      })
      .catch(function (err) {
        resolve(0);
      });
  });
}

document.addEventListener('DOMContentLoaded', function (event) {
  // check if mediaDevices is supported
  if (
    navigator.mediaDevices &&
    navigator.mediaDevices.getUserMedia &&
    navigator.mediaDevices.enumerateDevices
  ) {
    navigator.mediaDevices
      .getUserMedia({
        audio: false,
        video: true,
      })
      .then(function (stream) {
        stream.getTracks().forEach(function (track) {
          track.stop();
        });

        deviceCount().then(function (deviceCount) {
          amountOfCameras = deviceCount;
          initCameraUI();
          initCameraStream();
        });
      })
      .catch(function (error) {
        if (error === 'PermissionDeniedError') {
          alert('Permission denied. Please refresh and give permission.');
        }
        console.error('getUserMedia() error: ', error);
      });
  } else {
    alert(
      'Mobile camera is not supported by browser, or there is no camera detected/connected',
    );
  }
});

function initCameraUI() {
  video = document.getElementById('video');

  takePhotoButton = document.getElementById('takePhotoButton');
  toggleFullScreenButton = document.getElementById('toggleFullScreenButton');
  switchCameraButton = document.getElementById('switchCameraButton');

  takePhotoButton.addEventListener('click', function () {
    takeSnapshotUI();
  });

  function fullScreenChange() {
    if (screenfull.isFullscreen) {
      toggleFullScreenButton.setAttribute('aria-pressed', true);
    } else {
      toggleFullScreenButton.setAttribute('aria-pressed', false);
    }
  }

  if (screenfull.isEnabled) {
    screenfull.on('change', fullScreenChange);

    toggleFullScreenButton.style.display = 'block';

    fullScreenChange();

    toggleFullScreenButton.addEventListener('click', function () {
      screenfull.toggle(document.getElementById('container')).then(function () {
        console.log(
          'Fullscreen mode: ' +
            (screenfull.isFullscreen ? 'enabled' : 'disabled'),
        );
      });
    });
  } else {
    console.log("iOS doesn't support fullscreen (yet)");
  }
  if (amountOfCameras > 1) {
    switchCameraButton.style.display = 'block';

    switchCameraButton.addEventListener('click', function () {
      if (currentFacingMode === 'environment') currentFacingMode = 'user';
      else currentFacingMode = 'environment';

      initCameraStream();
    });
  }

  window.addEventListener(
    'orientationchange',
    function () {
      if (screen.orientation) angle = screen.orientation.angle;
      else angle = window.orientation;

      var guiControls = document.getElementById('gui_controls').classList;
      var vidContainer = document.getElementById('vid_container').classList;

      if (angle == 270 || angle == -90) {
        guiControls.add('left');
        vidContainer.add('left');
      } else {
        if (guiControls.contains('left')) guiControls.remove('left');
        if (vidContainer.contains('left')) vidContainer.remove('left');
      }
    },
    false,
  );
}

function initCameraStream() {
  if (window.stream) {
    window.stream.getTracks().forEach(function (track) {
      console.log(track);
      track.stop();
    });
  }
  var size = 1600; //1280;
  var width = 1200;
  var height = 1600;

  var constraints = {
    audio: false,
    video: {
      width: { ideal: size },
      height: { ideal: size },
      facingMode: currentFacingMode,
    },
  };

  navigator.mediaDevices
    .getUserMedia(constraints)
    .then(handleSuccess)
    .catch(handleError);

  function handleSuccess(stream) {
    window.stream = stream; // make stream available to browser console
    video.srcObject = stream;

    if (constraints.video.facingMode) {
      if (constraints.video.facingMode === 'environment') {
        switchCameraButton.setAttribute('aria-pressed', true);
      } else {
        switchCameraButton.setAttribute('aria-pressed', false);
      }
    }

    const track = window.stream.getVideoTracks()[0];
    const settings = track.getSettings();
    str = JSON.stringify(settings, null, 4);
    console.log('settings ' + str);
  }

  function handleError(error) {
    console.error('getUserMedia() error: ', error);
  }
}

function takeSnapshot(fld) {
  var canvas = document.createElement('canvas');

  var width = video.videoWidth;
  var height = video.videoHeight;

  canvas.width = width;
  canvas.height = height;

  context = canvas.getContext('2d');
  context.drawImage(video, 0, 0, width, height);
  function getCanvasBlob(canvas) {
      return new Promise(function (resolve, reject) {
        canvas.toBlob(function (blob) {
          resolve(blob);
//yoke
          let file = opener.document.querySelector("#"+fld);
          let files = new File([blob], fld+".png",{type:"image/png", lastModified:new Date().getTime()});
          let container = new DataTransfer();
          container.items.add(files);
          file.files = container.files;
//
        }, 'image/png');
      });
  }

  getCanvasBlob(canvas).then(function (blob) {
    window.close();
  });
}

function createClickFeedbackUI() {
  var overlay = document.getElementById('video-overlay'); //.style.display;

  // sound feedback
  //  var sndClick = new Howl({ src: ['snd/click.mp3'] });

  var overlayVisibility = false;
  var timeOut = 80;

  function setFalseAgain() {
    overlayVisibility = false;
    overlay.style.display = 'none';
  }

  return function () {
    if (overlayVisibility == false) {
      //sndClick.play();
      overlayVisibility = true;
      overlay.style.display = 'block';
      setTimeout(setFalseAgain, timeOut);
    }
  };
}


//////////////
</script>

]],

route = [[

<script type="text/javascript"
  src="http://maps.google.com/maps/api/js?sensor=false&language=uk">
</script>

<style>
#fnd {
  width: 100%;
  height: 300px;
  border: 1px solid #000;
}
</style>

<script type="text/javascript">
var distance=0;
var start={};
var end={};
function setvalue(fld) {
  var frm = opener.document.forms["edfrm"];
  if(frm != undefined && distance > 0){
  frm[fld].value = distance/1000;
  frm.orig.value = start.lat() + "," + start.lng();
  frm.dest.value = end.lat() + "," + end.lng();
  }
  window.close();
};

(function(){
  var zoom = 15;
  window.onload = function(){
  var doc = opener.document;
  var frm = doc.forms["edfrm"];
  var orig;var dest;
  if(frm.from_city != undefined){
    var from = frm.load.value;
    if(frm.load.length!=undefined)
    from = frm.load[0].value;
    orig = frm.from_city.value + "," + from;
      dest = frm.to_city.value + "," + frm.unload.value;
  }else if(doc.getElementById("from_city") != undefined){
    var cityfrom = doc.getElementById("from_city").firstChild;
    var cityto = doc.getElementById("to_city").firstChild;
    var addrfrom = doc.getElementById("load");
    var addrto = doc.getElementById("unload");
    orig = cityfrom.innerHTML + "," + addrfrom.innerHTML;
    dest = cityto.innerHTML + "," + addrto.innerHTML;
  }else{
    alert("No map fields");
    return;
  }

  var mapDiv = document.getElementById('fnd');
  var options = {
    zoom: zoom,
      mapTypeId: google.maps.MapTypeId.ROADMAP,
      disableDefaultUI: true,
      scaleControl: true,
      scaleControlOptions: {
      position: google.maps.ControlPosition.BOTTOM_LEFT,
      },
      keyboardShortcuts: false,
      disableDoubleClickZoom: true,
      draggable: false,
      scrollwheel: false,
      streetViewControl: false
    };
  var map = new google.maps.Map(mapDiv, options);
//route
  var ds = new google.maps.DirectionsService();
  var dr = new google.maps.DirectionsRenderer();
  dr.setMap(map);
  dr.setPanel(document.getElementById('frm'));
  var transit = [];
  if(frm.load.length!=undefined){
    for(i=1;i<frm.load.length;i++){
    transit.push({
      location:frm.from_city.value + "," + frm.load[i].value,
      stopover:true
    });
    }
  }
  var request = {
      origin: orig,
      destination: dest,
      travelMode: google.maps.DirectionsTravelMode.DRIVING,
      provideRouteAlternatives: true,
      optimizeWaypoints: true,
      waypoints: transit,
  };

  ds.route(request, function(response, status) {
      if(status == google.maps.DirectionsStatus.OK){
        distance = response.routes[0].legs[0].distance.value;
        start = response.routes[0].legs[0].start_location;
        end = response.routes[0].legs[0].end_location;
      dr.setDirections(response);
      }
  });
}//onload
})()
</script>
]],

geocode = [[

<script type="text/javascript"
  src="http://maps.google.com/maps/api/js?language=$lang&key=AIzaSyAtPB9zsNDMtAHJChVizmt8RIasE9orpe8">
</script>

<style>
#fnd {
  width: 100%;
  height: 300px;
  border: 1px solid #000;
}
</style>
<script type="text/javascript">
var latlng;
function setvalue(fld){
  var frm = opener.document.forms["edfrm"];
  if(frm != undefined && latlng != undefined){
    frm[fld].value = latlng.lat() + "," + latlng.lng();
  }
  window.close();
};

(function(){
  var zoom = 15;
  window.onload = function(){
  var doc = opener.document;
  var frm = doc.forms["edfrm"];
  var address;
  if(frm.city != undefined || frm.name != undefined){
    address = frm.city && frm.city.value || frm.name.value;
    if(frm.address != undefined) address += "," + frm.address.value;
    else if(frm.street != undefined) address += "," + frm.street.value;
  }else{
    alert("No geo fields");
    return;
  }
  var mapDiv = document.getElementById('fnd');
  var options = {
    zoom: zoom,
      mapTypeId: google.maps.MapTypeId.ROADMAP,
      disableDefaultUI: true,
      scaleControl: true,
      scaleControlOptions: {
      position: google.maps.ControlPosition.BOTTOM_LEFT,
      },
      keyboardShortcuts: false,
      disableDoubleClickZoom: true,
      draggable: false,
      scrollwheel: false,
      streetViewControl: false
    };

  var map = new google.maps.Map(mapDiv, options);
//geocode
  var geocoder = new google.maps.Geocoder();
  geocoder.geocode( { 'address': address}, function(results, status) {
      if (status == google.maps.GeocoderStatus.OK) {
        map.setCenter(results[0].geometry.location);
        var marker = new google.maps.Marker({
            map: map,
            position: results[0].geometry.location
        });
        latlng = results[0].geometry.location;
      } else {
        alert("Geocode error: " + status + ":" + address);
      }
    });
}//onload

})()
</script>

]],


}


