------------------------------
-- yoke © luamani@yahoo.com --
------------------------------

local db = {}

local insert,concat,format,gsub = table.insert,table.concat,string.format,string.gsub
local modified = false

local function blob(self,base,recno,field,data)

  if type(data) ~= "string" then return end
  local sql = concat{'update ',base, ' set ',field,' = ? where rowid = ',recno}
  local stmt = self.conn:prepare(sql)
  assert(stmt,self.conn:errmsg())
  if stmt then
    stmt:bind_blob(1,data)
    stmt:step()
    stmt:finalize()
  end

end

local function update(self,name,row,recno,out,blobs)
  local coma = ''
  local qry = concat{"update ",name," set "}
  for k,v in pairs(row) do
    if blobs[k] then
      blobs[k] = v
    else
      k = (k or ""):gsub("^.+%.","")
      if type(v) == 'string' then
        v = gsub(v,"'","''"):gsub("^%s+",""):gsub("%s+$","")
      end
      qry = concat{qry,coma,k," = '",v,"'"}
      coma = ","
    end
  end
  if coma == '' then return end
  qry = concat{qry," where rowid = ",recno}
  --assert(self.conn:exec(qry) == self.dbm.OK,self.conn:errmsg()) --todo locked
  if self.conn:exec(qry) ~= self.dbm.OK then
    insert(out,format("%s:%s\n",qry,self.conn:errmsg()))
  elseif self.conf.debug and self.user.admin then
    insert(out,format("update:%s,modified:%s",qry,modified))
  end
end

local function getblob(row,fld)

  local fname = "noname"
  local ftype = "application/x-unknown"
  local fsize = nil

  local content = nil
  local filename = params[fld.file]

  if filename then
    if type(filename) == "table" then -- cgi
      fname = filename.name or "noname"
      fsize = filename.size
      ftype = filename['content-type']
      content = filename.contents
    end
  elseif params.content and params.ftype then --ajax update --todo test
    local base64 = require("ee5_base64")
    content = base64.decode(params.content)
    ftype = params.ftype
    --row.type = ftype
    fname = params.fname
    fsize = string.len(content) --params.fsize
    --row.fsize = fsize
  end
  if fsize then
    if fname then row[fld.file] = fname end
  end

  if row[fld.file] == '' then row[fld.file] = nil end

  if not content or content == '' then
    return
  end

  if content and type(content) == 'string' then
    row[fld[1]] = content
  end

end

function db:run(recno,out,subname,subform,subrecno,shift)
  local recno = tonumber(recno) or 0
  if recno < 0 then return 0 end
  local dform = subform or self.def.view
  local dname = subname or self.def.view.mod or self.model
  local fpref = ""
  if subname then
    fpref = dname
    subrecno = tonumber(subrecno) or 0
    if subrecno > 0 then recno = subrecno end
  end
  local row = {}
  local qry,coma,rec  = '','',{}
  if not out then out = {} end
  if recno > 0 and subrecno ~= 0 then
    qry = concat{"select rowid,* from ",dname," where rowid=",recno}
    for nrow in self.conn:nrows(qry) do
      if nrow.user then
        if not self:access(dname,recno,nrow.user,"update") then
          insert(out,concat{lang["No access update"] or "No access update"," ",dname})
          return
        elseif nrow.user ~= tonumber(params.user or 0) and
              not self:access(dname,recno,nrow.user,"assign") then
          params.user = nrow.user
        end
      end
      rec = nrow
    end
  elseif not self:access(dname,nil,self.user.id,"new") then
    insert(out,concat{lang["No access create"] or "No access create"," ",dname})
    return
  end

  local blobs = {}
  for k,v in ipairs(dform) do
    if not ({SHOW=1,OBJECT=1})[v.ht] and (v[2] or v.ht == "CHECKBOX") then
      local vid = v[1]
      local fld = concat{fpref,v.fd or vid}
      local str = params[fld]
      if (v.ht == "REF" or v.ht == "SELF") then
        if type(str) == "table" and shift then
          if str[shift] ~= "" then str = params[fld..(v.fd and "Id" or "")] end
        elseif str and str ~= "" then
          str = params[fld.."Id"] or ""
        end
      end
      if type(str) == "table" then str = shift and str[shift] or str[1] or "" end
      if v.ht == "DATE" or v.ht == "DATETIME" then str = self.util:stime(str) or ''
      elseif v.ht == "REAL" or v.ht == "GEO" then str = gsub(tostring(str),",",".")
      elseif v.ht == "LIST" then
        local list = type(params[vid]) == "table" and params[vid] or {params[vid]}
        local vals = {}
        for k,val in ipairs(list) do insert(vals,1,val) end
        str = concat(vals,";")
      elseif v.ht == "MATRIX" then
        local rows = type(params[fld]) == "table" and params[fld] or {params[fld]}
        local value = {}
        local len = #v.rx
        for _,y in ipairs(rows) do
          local val = {}
          for i,x in ipairs(v.rx) do
            if params[concat{y,"-",x[1]}] == '1' then insert(val,x[1]) end
          end
          if #val < len then insert(value,concat{y,":",concat(val),#val == 0 and "0" or ""}) end
        end
        str = concat(value,",")
      end
      local id = (v.fd or vid or ""):match("%.?([^%.]+)$")
      if params[fld] and tostring(rec[id]) ~= tostring(str) then
        modified = true
        row[id] = str
      end
    elseif v.ht == "BLOB" then
      blobs[v[1]] = true
      if dform.file and v.file then --get blob from post
        getblob(row,v)
      end
    end

    if (v[1] or ""):match("%.?user") then
      local user,time = self.user.id,os.time()
      row.editor = user
      row.edited = time
      if recno == 0 or subrecno == 0 then
        row.user = tonumber(params.user) or user
        row.creator = user
        row.created = time
      end
    end
  end

  if subname and recno > 0 and dform.link and dform.link.id and subrecno == 0 then
    row[dform.link.id] = recno
  end

  if self.def.update and not subname then
    local mes = self.def:update(row,recno)
    if mes then
      insert(out,mes)
      return
    end
    if next(row) then modified = true end
  end
  --
  if (recno == 0 or subrecno == 0) then
    local flds,vals = "",""
    qry = concat{"insert into ",dname," (%s) values(%s)"}
    for k,v in pairs(row) do
      if blobs[k] then
        blobs[k] = v
      else
        k = (k or ""):gsub("^.+%.","")
        flds = concat{flds,coma,k}
        if type(v) == 'string' then
          v = gsub(v,"'","''"):gsub("^%s+",""):gsub("%s+$","")
        end
        vals = concat{vals,coma,"'",v,"'"}
        coma = ","
      end
    end
    qry = format(qry,flds,vals)
    if self.conf.debug and self.user.admin then insert(out,format("new:%s",qry)) end
    local ret = self.dbm.OK --self.conn:exec(qry)

    assert(self.conn:exec(qry) == self.dbm.OK,concat{self.conn:errmsg(),":",qry}) --todo check

    if ret ~= self.dbm.OK then
      local mes = self.conn:errmsg()
      if self.conf.cli then print(mes,":",qry) return 0 end
      insert(out,format("new:%s:%s",qry,mes))
    else
      local newrec =  self.conn:last_insert_rowid() -- !!!
      if newrec > 0 then
        if subname and recno > 0 and shift then
          local sql = concat{"update ",dname," set ",shift," = ",newrec," where rowid = ",recno}
          self.conn:exec(sql)
        else
          recno = newrec
          params.recno = recno
        end
        for k,v in ipairs(dform) do
          if v.ht == "OBJECT" and v[2] then
            self:run(newrec,out,v.rt,v.rf,0,v[1])
          end
        end
      end
    end
  elseif modified then --update
    for k,v in ipairs(dform) do
      if v.ht == "OBJECT" and v[2] then
        self:run(recno,out,v.rt,v.rf,params[v.rt .. "_recno"],v[1])
        params[v.rt .. "_recno"] = nil
      end
    end
    if not (params.list and not subname) then
      update(self,dname,row,recno,out,blobs)
    end
  end
--
  if params.list and not subname then
    local list,frm = params.list,nil
    dform.list = self.util.rels(self,dname,dform.list)
    for i,line in ipairs(dform.list or {}) do if line[1] == list then frm = line[2] break end end
    if type(params[list]) == "string" then params[list] = {params[list]} end

    if frm and type(params[list]) == "table" then
      self.conn:exec("begin")
      local len = #params[list]
      for i,item in ipairs(params[list]) do
        if tonumber(item) then -- avoid newtr
          self:run(recno,out,list,frm,item,i)
        end
      end
      self.conn:exec("commit")
      if modified then
        if self.def.update then
          local mes = self.def:update(row,recno)
          if mes then
            insert(out,mes)
            return
          end
        end
        update(self,dname,row,recno,out,blobs)
      end
    end
    params.list = nil
    modified = false
  end

  for field,data in pairs(blobs) do
    blob(self,dname,recno,field,data)
  end

  return recno
end

return db
