------------------------------
-- yoke © luamani@yahoo.com --
------------------------------

local concat = table.concat
local response = require"yoke.snap.http.response"
local request = require"yoke.snap.http.request"
local codes = require"yoke.snap.http.status"
table.class = table.class or require'class'

local http = {}

function http.app(env,app)
  local hdrs = {["Content-Type"] = "text/html"}
  local req = request(env)
  local res = response(200,hdrs)

  params = req.params

  if app.conf.autologin then
    cookie = function(key,val)
        local fd = io.open(concat{app.conf.db,".cli"},val and "w" or "r")
        if not fd then return end
        if val then
          fd:write(concat{key,"=",val})
          fd:close()
        else
          local line = fd:read("*L")
          local k,v = line:match("(%w+)=(%w+)")
          if k == key then return v end
        end
    end
  else
    cookie = function(key,val)
      if val then
        res:cookie(key,val)
      else
        return req.cookies[key]
      end
    end
  end

  header = function(key,val)
    if val then
      res.headers[key] = val
    else
      return res.headers[key]
    end
  end
  local page = app(req.env):site()

  local function site()
    coroutine.yield(page)
  end

  if type(page) == 'string' then
    hdrs['Content-Length'] = page:len()
  end

  return 200, hdrs, coroutine.wrap(site)

end


function http.content(out, page, func)
  local write = out[func or "write"]
  local flush = out.flush
  local ok, res = xpcall(page, debug.traceback)
  while ok and res do
    write(out, res)
    if flush then flush(out) end
    ok, res = xpcall(page, debug.traceback)
  end
  write(out,'\r\n')
  if not ok then
    write(out,concat{"ERROR:",tostring(res)})
  end
end

function http.send(out, status, headers, page, func, httpline)
   local write = out[func or "write"]
   if type(status) == "number" or status:match("^%d+$") then
     status = concat{status," ",codes[tonumber(status)]}
   end
   if httpline then
     write(out, "HTTP/1.1 " .. (status or "500 Internal Server Error") .. "\r\n")
   else
     write(out, "Status: " .. (status or "500 Internal Server Error") .. "\r\n")
   end
   for h, v in pairs(headers or {}) do
      if type(v) ~= "table" then
         write(out, h .. ": " .. tostring(v) .. "\r\n")
      else
         for _, v in ipairs(v) do
            write(out, h .. ": " .. tostring(v) .. "\r\n")
         end
      end
   end
   write(out, "\r\n")
   http.content(out, page, func)
end

local function iter(msg)
  local sent = false
  return function()
           if sent then return nil
           else
             sent = true
             return msg
           end
         end
end

function http.errors(msg,opt,status)
  if status == 404 then
    status = "404 Not Found"
  else
    local write = opt.errors["write"]
    write(opt.errors, "Error: " .. tostring(msg) .. "\n")
    status = "500 Internal Server Error"
  end
  local msg = tostring(msg)
  local headers, page = {
        ["Content-Type"] = "text/plain",
        ["Content-Length"] = #msg
      },iter(msg)
  http.send(opt.output, status, headers, page, opt.write,opt.httpline)
  return status, headers
end

function http.input(io,func)
  local input = {}
  local read = io[func or "read"]
  function input:read(len)
    len = len or self.length or 0
    if len > 0 then return read(io, len) end
  end
  return input
end

function http.idx(env)
  if type(env) == "table" then
    return function (tbl, key)
             local val = env[key]
             tbl[key] = val or ""
             return val or ""
           end
  else
    return function (tbl, key)
             local val = env(key)
             tbl[key] = val or ""
             return val or ""
           end
  end
end

function http.env(opt)
  local env = {}
  setmetatable(env, { __index = http.idx(opt.env) })
  env.input = http.input(opt.input, opt.read)
  env.errors = opt.errors
  env.input.length = tonumber(env.CONTENT_LENGTH) or 0
  if env.PATH_INFO == "" then env.PATH_INFO = "/" end
  return env
end

function http.run(app,opt)
  local env = http.env(opt)
  local ok, status, headers, page = xpcall(function () return http.app(env,app) end,
                 function (msg)
                    if type(msg) == "table" then
                       env.STATUS = msg[1]
                       return tostring(msg[2])
                    else
                       return debug.traceback(msg,2)
                    end
                 end)
  if ok then
    if not headers["Content-Length"] then
      if opt.httpline then --todo
        headers["Transfer-Encoding"] = "chunked"
        local unchunked = page
        page = function()
                local msg = unchunked()
                  if msg then
                    return string.format("%x\r\n%s\r\n", #msg, msg)
                  end
               end
      end
    end

    http.send(opt.output, status, headers, page, opt.write, opt.httpline)
  else
    return http.errors(status,opt,env.STATUS)
  end
end

return function(app,opt)
  http.run(app,{
    input =  opt and opt.input or io.stdin,
    output = opt and opt.output or io.stdout,
    read = opt and opt.read,
    write = opt and opt.write,
    errors = io.stderr,
    env = opt and opt.env or os.getenv,
    httpline = opt and opt.httpline or false, --todo test HTTP/1.1 404
  })
end
