------------------------------
-- yoke © luamani@yahoo.com --
------------------------------

local concat,insert = table.concat,table.insert

local db = {}

function db:run()
  --head
  local ok,def = pcall(require,"yoke.snap.head")
  if ok then def(self) end
  for _,app in ipairs(self.conf.apps or {}) do
    ok,def = pcall(require,concat{"apps.",app[1],".meta.head"})
    if ok and type(def) == 'function' then
      def(self)
    end
  end
  self.conn:exec("begin")
  --site
  for _,app in ipairs(self.conf.site or {}) do
    ok,def = true,''
    local conf = require(concat{"yoke.site.apps.",app[1],".conf"}) or {}
    for i,row in ipairs(conf) do
      local mod = require(concat{"yoke.site.apps.",app[1],".mods.",row[1]})
      if mod and mod.view and not mod.view.mod then
        for _,cnf in pairs(self.conf.apps) do
          local view = cnf.extend and cnf.extend[row[1]] and cnf.extend[row[1]].view
          if view then
            for _,fld in ipairs(view) do
              table.insert(mod.view,fld)
            end
          end
        end
        ok,def = self:create(row[1],mod.view)
      end
      if not ok then print("init:",def,"<br/>\n") break end
    end
  end
  --apps
  for _,app in ipairs(self.conf.apps or {}) do
    if not ok then break end
    local conf = app[2] and require(concat{"apps.",app[1],".conf"}) or {}
    for i,row in ipairs(conf or {}) do
      local id = row[1]
      local mod = require(concat{"apps.",app[1],".mods.",id})
      if mod and mod.view and not mod.view.mod then

        setmetatable(mod,{__index=self})
        if mod.field and mod.field[1] then mod.field[1](mod) end -- apps fields
        local ext = conf.extend and conf.extend[id] or nil
        if ext then
          if ext.field and ext.field[1] then ext.field[1](mod) end
          if ext.view then --todo
            for k,v in ipairs(ext.view) do
              table.insert(mod.view,v)
            end
          end
        end

        ok,def = self:create(row[1],mod.view,app[1])
        --setmetatable(mod,nil) -- prevent loop
        if not ok then
          print("create:",row[1],",",def,"<br/>")
          break
        end
      end
    end
  end
  --init
  if ok then
    --local qry = "select nullif(count(*)=0,0) from sqlite_master where type='table' limit 1"
    local qry = "select nullif(count(*)=0,0) from users where login='admin' limit 1"
    for usr in self.conn:urows(qry) do
      if self.conn:exec(concat{"insert into users (rowid,login,pw,admin,active)",
        "values(1,'admin','admin',1,1)"}) == 1 then
        print("init users:",self.conn:errmsg(),"<br/>")
        return
      end
    end
    ok,def = pcall(require,"yoke.snap.tail")
    if ok then def(self) end
    for _,app in ipairs(self.conf.apps or {}) do
      ok,def = pcall(require,concat{"apps.",app[1],".meta.tail"})
      if ok then if type(def) == 'function' then def(self)
        elseif type(def) == 'table' and def[1] then self:migrate(def,app[1]) end
      end
    end
    self.conn:exec("commit")
  else
    print('init:false')
  end
end

function db:create(name,view,app)
  local types = setmetatable({
  ["SELF,REF,CHECKBOX,DATE,DATETIME,OBJECT,MAP,NUM"] = 'INT',
  ["NUMBER,RANGE"] = 'NUMERIC',
  ["SELECT,PW,LIST,MULTI,TEXTAREA,HTML,EMAIL,WWW,FILE,GEO,MATRIX,LINK"] = 'TEXT',
  },{__index=function(t,key) for k,v in pairs(t) do if k:find(key)
                             then return v end end return (key or 'TEXT') end}) --todo select int
  local sub = name:find("%.")
  local base = app or "main"
  local tbl = name
  if sub then
    base,tbl = name:match("(.*)%.(.*)")
  end
--
  local sql = concat{'pragma ',base,'.','table_info(',tbl,')'}

  local info,alter = {},{}
  for rows in self.conn:rows(sql) do
    info[rows[2]] = {def=rows[3]} -- name = {type}
  end
--
  local ids = {}
  sql = {}
  for k,v in ipairs(view) do
    local id = v[1]
    sub = id:find("%.")
    if sub then if id:sub(1,sub-1) == name then id = id:sub(sub) else id = nil end end
    if id and v.ht ~= "SHOW" then
      local affinity = v.ht and types[v.ht] or 'TEXT'
      if v.ht == 'SELECT' and v.rt then affinity = 'INT' end
      local field = concat{id," ",affinity," ",string.upper(v.constraint or "")}
      if not info[id] or v.alter then alter[#alter+1] = v end -- for alter
      if not ids[id] then ids[#ids+1] = id end -- for create view
      insert(sql,field)
      if id == 'user' and v.ht == "SELECT" then
        insert(sql,"creator INT,created INT,editor INT,edited INT")
        ids[#ids+1] = "creator"
        ids[#ids+1] = "created"
        ids[#ids+1] = "editor"
        ids[#ids+1] = "edited"
      end
    end
    if id and v.ht == "OBJECT" then
      self:create(v.rt,v.rf)
    end
  end
--
  if not next(info) then
    local qry = concat{"create table if not exists ",base,'.',tbl," (rowid INTEGER PRIMARY KEY,",
    concat(sql,","),") "}
    if view.view then
      local select = concat{" as select rowid,",concat(ids,","), " from ", view.view[1], view.view[2]}
      qry = concat{"create view if not exists ",base,'.',tbl," (rowid,",
      concat(ids,","),") ",select}
    end
    if self.conn:exec(qry) ~= 0 then
      return false,concat{qry,":",self.conn:errmsg()}
    else
      return true,""
    end
  else
    local remove = {}
    for k,v in ipairs(alter) do
      if v.alter then
        if type(v.alter) == 'string' then
          self.conn:exec(concat{"alter table ",base,'.',tbl," rename ",v[1]," to ",v.alter})
        else
          remove[v[1]] = true
        end
      elseif v.ht ~= 'SHOW' then
        local affinity = v.ht and types[v.ht] or 'TEXT'
        if v.ht == 'SELECT' and v.rt then affinity = 'INT' end
        self.conn:exec(concat{"alter table ",base,'.',tbl," add column ",v[1]," ",affinity})
      end
    end
    if next(remove) then
      local flds = {}
      for k,v in ipairs(sql) do
        local toremove = false
        for kk,vv in pairs(remove) do
          if type(kk) == 'string' and v:match(concat{kk,' '}) then
            toremove = true
          end
        end
        if not toremove then table.insert(flds,v) end
      end
      if #flds > 0 then
        local qry = concat{"create table if not exists ",base,'.',tbl,"_ (rowid INTEGER PRIMARY KEY,",
        concat(flds,","),") "}
        if self.conn:exec(qry) == 0 then
          if self.util.copy(self,base,base,tbl,concat{tbl,'_'}) == 0 then
            self.conn:exec(concat{'drop table ',base,'.',tbl})
            self.conn:exec(concat{'alter table ',base,'.',tbl,'_ rename to ',tbl})
          end
        end
      end
    end
    return true,""
  end
end

function db:migrate(rows,app,tab)
  local num = 0
  function record(row)
    num = num + 1
    local sql,mod,id = {},tab or row[1],tab and row[1] or row[2]
    local vals,flds,bind = {id},{'rowid'},{'?'}
    for k,v in pairs(row) do
      if type(k) == 'string' then
        flds[#flds+1] = k
        bind[#bind+1] = '?'
      end
      if type(v) == 'table' then
        vals[#vals+1] = record(v)
      elseif type(k) == 'string' then
        vals[#vals+1] = v
      elseif k == 2 and id == 0 then
        vals[1] = num
      end
    end
    local base = app or 'main'
    if not mod:match('%.') then
      mod = concat{base,'.',mod}
    end
    sql = {'insert into',mod,'(',concat(flds,','),')','values(',concat(bind,','),')'}
    for row in self.conn:urows(concat{'select rowid from ',mod,' where rowid=',id}) do
      local coma = ''
      sql = {'update',mod,'set'}
      for k,v in ipairs(flds) do sql[#sql+1] = coma ..v..' = ?'; coma = ',' end
      sql[#sql+1] = ' where rowid = ' .. id
    end
    local stmt = self.conn:prepare(concat(sql,' '))
    if stmt then
      stmt:bind_values(table.unpack(vals))
      stmt:step()
      stmt:finalize()
    end
    return id
  end

  for mod,row in pairs(rows) do
    if type(row) == 'function' then
      local ok,mess = pcall(row,self)
      if not ok then
        debug.log(mess)
      end
    elseif type(row) == 'table' and (type(row[1]) == 'string' and row[2] or
           type(row[1]) == 'number' and not row[2]) then
      record(row)
    elseif type(row) == 'table' then
      self:migrate(row,app,mod)
    end
  end
end

return db

