-----------------------
-- yoke © yokerp.com --
-----------------------

local db = {}
db.url = {}

local insert,concat,format = table.insert,table.concat,string.format

function db:mkqry(recno,model,form,edit)
  if not recno or not form then return "" end
  local qry = {"select ",model,".rowid"}
  local join = {}
  local user = false
  for k,v in ipairs(form) do
    local vid = (v[1] or ""):find("%.") and v[1] or concat{model,'.',v[1]}
    if v.ht ~= "SHOW" and v.ht ~= "OBJECT" and not (v.readonly and edit) then
      insert(qry,concat{",",vid})
      if v[1] == "user" then user = true end
    end
  end
  if user then insert(qry,concat{",",model,".created,",model,".edited",
    ",users.login as owner,editors.login as editor,creators.login as creator"})
  end
  insert(qry,concat{" from ",model})
  for k,v in pairs(form.join or {}) do insert(qry," left join " .. v) end

  if user then insert(qry,concat{" left join users on ",model,".user = users.rowid ",
    "left join users as creators on ",model,".creator = creators.rowid ",
    "left join users as editors on ",model,".editor = editors.rowid"})
  end

  insert(qry," where ")
  insert(qry,form.where or concat{model,".rowid=",recno})
  return concat(qry)
end

function db:stime(date)
  if not date then return end
  local y,m,d,h,min = date:match("(%d+)-(%d+)-(%d+)%s*(%d*):*(%d*)")
  if y and m and d then
    if (not min or min == '' or not h or h == '') then
      return os.time{year=y,month=m,day=d,hour=0,min=0,sec=0}
    else
      return os.time{year=y,month=m,day=d,hour=h,min=min,sec=0}
    end
  end
end

function db:sub(text,vals)
  if not text then return "" end
  text = text:gsub("%$([%d%w_-]+)(%$?)",function (x,y)
    if y == '$' then  x=x:gsub("[-_]"," ") return lang[x] or x
    elseif x then return vals[x]
    end
  end)
  return text
end

function db:escape(text) 
  if not text then return "" end
  if type(text) == "number" then text = tostring(text) end
  return text:gsub("[&><'\"]",function (x)
    if x == "&" then return "&amp;" 
    elseif x == "<" then return "&lt;"
    elseif x == ">" then return "&gt;"
    elseif x == '"' then return "&quot;"
    elseif x == "'" then return "&#39;"
--  elseif x == '\xa0' then return "&nbsp;"
    end 
  end)
end

-- RFC 2396
function db.url.decode(str)
  if not str then return nil end
  str = string.gsub (str, "+", " ")
  str = string.gsub (str, "%%(%x%x)", function(h) return string.char(tonumber(h,16)) end)
  str = string.gsub (str, "\r\n", "\n")
  return str
end

function db.url.encode(str)
  if not str then return nil end
  str = string.gsub (str, "\n", "\r\n")
  str = string.gsub (str, "([^%w ])",
        function (c) return string.format ("%%%02X", string.byte(c)) end)
  str = string.gsub (str, " ", "+")
  return str
end

function db.url.norm(path)
  local value = string.gsub(path ,'\\', '/')
  value = string.gsub(value, '^/*', '/')
  value = string.gsub(value, '(/%.%.?)$', '%1/')
  value = string.gsub(value, '/%./', '/')
  value = string.gsub(value, '/+', '/')

  while true do
    local first, last = string.find(value, '/[^/]+/%.%./')
    if not first then break end
    value = string.sub(value, 1, first) .. string.sub(value, last + 1)
  end

  while true do
    local n
    value, n = string.gsub(value, '^/%.%.?/', '/')
    if n == 0 then break end
  end

  while true do
    local n
    value, n = string.gsub(value, '/%.%.?$', '/')
    if n == 0 then break end
  end

  return value
end

function db.tohex(num)
  local chars = { '0','1','2','3','4','5','6','7','8','9','a','b','c','d','e','f' }
  local hex = {}
  repeat
    insert(hex, 1, chars[num % 16 + 1])
    num = math.floor(num / 16)
  until num == 0
  return concat(hex)
end

local b='ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/'

function db.unbase64(data)
    return ((data:gsub('.', function(x) 
        local r,b='',x:byte()
        for i=8,1,-1 do r=r..(b%2^i-b%2^(i-1)>0 and '1' or '0') end
        return r;
    end)..'0000'):gsub('%d%d%d?%d?%d?%d?', function(x)
        if (#x < 6) then return '' end
        local c=0
        for i=1,6 do c=c+(x:sub(i,i)=='1' and 2^(6-i) or 0) end
        return b:sub(c+1,c+1)
    end)..({ '', '==', '=' })[#data%3+1])
end

local bs = { [0] =
   'A','B','C','D','E','F','G','H','I','J','K','L','M','N','O','P',
   'Q','R','S','T','U','V','W','X','Y','Z','a','b','c','d','e','f',
   'g','h','i','j','k','l','m','n','o','p','q','r','s','t','u','v',
   'w','x','y','z','0','1','2','3','4','5','6','7','8','9','+','/',
}

function db.base64(s)
   local byte, rep = string.byte, string.rep
   local pad = 2 - ((#s-1) % 3)
   s = (s..rep('\0', pad)):gsub("...", function(cs)
      local a, b, c = byte(cs, 1, 3)
      return bs[a>>2] .. bs[(a&3)<<4|b>>4] .. bs[(b&15)<<2|c>>6] .. bs[c&63]
   end)
   return s:sub(1, #s-pad) .. rep('=', pad)
end

function debug.log(...)
  if not arg.conf.log then return end
  if arg.conf.cli then
    if type(...) ~= 'table' then
      print(...)
    else
      for key,val in pairs(...) do
        print(key,val)
      end
    end
  else
    for key,msg in pairs(table.pack(...)) do
      if key ~= 'n' then
        io.stderr:write(tostring(msg),' ')
      end
    end
    io.stderr:write('\n')
  end
end

function table.idx(tab,id,value)
  for i,val in ipairs(tab) do
    if id == val[1] then
      if type(value) == 'table' and tab[i-1] and value[1] ~= tab[i-1][1] then
        table.insert(tab,i,value)
        return
      else
        return i
      end
    end
  end
end

function db.conf(conf)
  for _,app in ipairs(conf.apps) do --index apps
    local cnf = concat{'apps.',app[1],'.conf'}
    package.loaded[cnf] = nil --todo force reload
    conf.apps[app[1]] = app[2] and require(cnf) or {}
    conf.apps[app[1]].mods = {}
    for _,line in ipairs(conf.apps[app[1]]) do --index mods
      conf.apps[app[1]].mods[line[1]] = line   --line = {'mod','Name',...}
    end
    if app[2] and not (params or {}).mod and conf.lang and conf.lang ~= 'en' then
      pcall(require,concat{"apps.",app[1],".lang.",conf.lang})
    end
  end
  for key,app in pairs(conf.apps) do --extend conf
    for id,ext in pairs(app.extend or {}) do
        if ext.conf then
          debug.log('EXTEND CONF:',key,",",id)
          for name,val in pairs(ext.conf) do
            for i,line in ipairs(conf.apps[conf.mods[id]] or {}) do
              if line[1] == id then
                line[name] = val --debug.log("EXTEND ",id,":",name,":",val)
              end
            end
          end
        end
    end
  end
  return conf
end

function db.link(self,mod,name,qry,opts,li,loading)
  if self.conf.cli then return name end
  local css = nil
  local popup = mod == 'popup' and true or nil
  if type(opts) == 'table' then
    if opts.css then css = opts.css end
    if opts.popup then popup = true end
  elseif opts then
    css = opts
  end
  if not (mod or ''):match('%.') and self.conf.apps[self.app]
                                 and self.conf.apps[self.app].mods
                                 and self.conf.apps[self.app].mods[mod] then --todo base
    mod = concat{self.app,'.',mod}
  end
  local href = [[%s<a class="%s" href="%s" onclick="%s;return false;">%s</a>]]
  local get,json,js = {},{},''
  if mod ~= 'menu' then
    for k,v in pairs(qry or {}) do
      get[#get+1] = concat{k,"=",tostring(v)}
    end
    for k,v in pairs(qry or {}) do json[#json+1] = concat{"'",k,"':'",tostring(v),"'"} end
  end
  json = concat(json,',')

  if popup then
    if mod ~= 'popup' then
      get[#get+1] = concat{'mod=',mod}
    end
    js = format("javascript:popup(%s)",concat{"'",concat(get,'&amp;'),"'"})
  elseif mod == 'menu' then
    js = format("javascript:dropdown(this)")
  else
    js = format("javascript:acall('%s'%s%s)",mod,qry and concat{",{",json,"}"} or "",
                                              loading and concat{",'",loading,"'"} or '')
  end

  if mod ~= 'menu' then
    get = concat{self.conf.idx,popup and '?popup=' or "?mod=",mod or '',
                  qry and concat{"&",concat(get,'&'),"&lynx=true"} or "&lynx=true"}
  end


  if css == "button" or css == "submit" then
    local form = li and format("form='%s'",li) or ''
    return format([[<button type="%s" name="act" value="%s"
                      onclick="%s;return false;" %s>%s</button>]],
                      css,qry and qry.act or '',js,form,name)
  elseif css == 'link' then
    return format(href,'','nodeco',get,js,
              format("<input type='button' value='%s'/>",name))
  end
  --
  if mod == "menu" then
    local menu = {}
    for _,item in ipairs(qry or {}) do
      insert(menu,self:link(item[1],item[2],item[3],'dropdown',nil,item.loading))
    end
    local link = concat{format(href,li or '','nodeco','#',js,name),
      [[<div class='dropdown' style="display:none;">]],concat(menu),[[</div>]]
    }
    if #menu == 0 then
      link = concat{"<a class='nodeco' style='text-decoration:none'>",name,"</a>"}
    end
    return link
  end
  --
  return name and format(href,li or '',css or '',get,js,name) or js
end

function db.apps(self,val) --check if app in val
  function clean(tbl,key)
    if not tbl then return end
    for k=key or 1,#tbl do
      local app = nil
      if type(tbl[k]) == 'table' then
        app = tbl[k].rt and tbl[k].rt:match("([^%.]+)%.")
      elseif type(tbl[k]) == 'string' then
        app = tbl[k]:match("^([^%.%s]+)%.")
      end
      if app and not self.conf.apps[app] then
        debug.log('REMOVE FIELD',app,tbl[k][1] or tbl[k])
        if table.remove(tbl,k) then
          clean(tbl,k)
        end
      end
    end
  end

  local apps = true
  if not val then
    if self.def then
      local view = self.def.view
      local list = self.def.list
      local where = list and list.where
      local join = list and list.join
      clean(view)
      clean(list)
      clean(join)
      clean(where)
    end
  elseif type(val) == 'string' then
    if val:match("%.") then
      local app = val:match("([^%.]+)")
      if not self.conf.apps[app] then apps = false end
    end
  elseif type(val) == 'table' then
    if val.ls and val.ls.apps then
      if type(val.ls.apps) == 'string' then
        if not self.conf.apps[val.ls.apps] then apps = false end
      elseif type(val.ls.apps) == 'table' then
        for _,a in ipairs(val.ls.apps) do
          if not self.conf.apps[a] then apps = false end
        end
      end
    else
      clean(val)
      if val.join then
        local join = val.join
        clean(join)
      end
    end
  end
  return apps
end

function db.rels(self,mod,list,smod,recno,appl)

  local list = list or {}
  local rels,dels = {},{}
  local app = appl
  if list.done and not smod and not recno then return list end
  if not app then
    if (mod or ""):match("%.") then
      app,mod = mod:match("([^%.]+)%.(.*)")
    elseif self.app then
      app = self.app
    end
    if (smod or ""):match("%.") then
      _,smod = smod:match("([^%.]+)%.(.*)")
    end
  end
  local conf = self.conf.apps[app]
  for k,v in ipairs(conf or {}) do
    if v[3] and (v[1] == mod or concat{app,'.',v[1]} == mod) then --v[3] is app rels
      rels = v -- {mod={id='refid',mod='refbase'},...}
    end
  end
  if smod then
    local rel = rels[3] and rels[3][smod] or {}
    if rel.ls then return rel.ls end
    for k,v in ipairs(conf or {}) do --{smod,Name,mod={1,id=refid,mod=refbase}}
      if v[1] == smod and v.ls and db.apps(self,v) then
        if type(v[smod]) == 'table' and v[smod].id then
          v.ls.link = v[smod]
        elseif rel.id then
          v.ls.link = rel
        end
        v.ls.id = v[1] -- v[2] todo
        --todo sort from tabs
        if mod:match('%.') then
          mod = mod:match("%.([^%.]+)")
        end
        if type(v[mod]) == 'table' and v[mod].id then 
          v.ls.link = v[mod]
        end
        --
        return v.ls
      end
    end
    if rel then -- {smod='app'}
      local app = type(rel) == 'table' and rel[1] or rel
      local conf = self.conf.apps[app]
      for k,v in ipairs(conf or {}) do
        if v[1] == smod and v.ls then
          if self.conf.lang and self.app ~= app then
            require(concat{"apps.",app,".lang.",self.conf.lang})
          end
          if type(v[mod]) == 'table' and v[mod].id then
            v.ls.link = v[mod]
          elseif rel.id then
            v.ls.link = rel
          end
          --
          v.ls.id = v[1]
          --
          return v.ls
        end
      end
    end
    return
  end
  for id,row in pairs(rels[3] or {}) do
    --{'rel',rel='app',rel={'app'},rel={'app',10},rel={'app',1,id},rel={id=fld}}
    local name,idx,link,rel,app = row,1,nil,nil,row --todo test
    --local name,idx,link,rel = row,1,nil,nil
    if type(row) == 'table' and not row.id then
      app,idx,link = row[1],row[2] or idx,row[3]
    end
    if type(id) == "number" or row.id then -- the same app {'rel',rel={id=fld}}
      if row.id then name = id end
      --app = self.app --todo test
      app = appl or self.app --appl or app or self.app
      for k,v in ipairs(conf or {}) do
        if v[1] == name and v.ls and db.apps(self,v) then
          if type(rels[name]) == 'table' and rels[name].id then
            v.ls.link = rels[name]
          end
          if row.id then v.ls.link,idx = row,row[1] or 1 else idx = id end
          rel = v
          if row.ls then rel.ls = row.ls end
          break
        end
      end
    elseif self.conf.apps[app] then -- linked app
      local conf = self.conf.apps[app]
--    if type(conf[1]) == 'function' then conf[1](self) end
      if self.conf.lang then
        pcall(require,concat{"apps.",app,".lang.",self.conf.lang})
      end
      for k,v in ipairs(conf or {}) do
        if v[1] == id and v.ls and db.apps(self,v) then
          if link then v.ls.link = link end
          name = id
          rel = v break
        end
      end
    end
--
    if rel then
      local ls = row.ls or rel.ls
      ls.id = rel[2] -- name
      if type(rel[name]) == 'table' and rel[name].id then 
          ls.link = rel[name]
      end
      if recno then
        local ids,where = ls.link.id,'('
        if type(ids) == 'string' then where = concat{ids,'=',recno}
        elseif type(ids) == 'table' then
          local Or = ''
          for _,w in ipairs(ids) do where = concat({where,Or,w,'=',recno},' ') Or = 'or' end
          where = concat{where,' ) '}
        end
        if ls.link.mod then where = concat{where,concat{" and ",ls.link.mod,"='",mod,"'"}} end
        if not ls.show then
          insert(dels,{mod=name,where=where,app=app})
        end
      else
        if not self.conf.site[app] then
          name = concat{app,'.',name} --todo base
        end
        list[#list+1] = {name,ls,idx,app}
      end
    end
  end

  if recno then return dels end

  table.sort(list,function(t1,t2) return t1[3] < t2[3] end)
  list.done = true
  return list
end

function db.copy(self,from,app,tab,to)
    local flds = {}
    for row in self.conn:nrows(table.concat{"pragma ",app,".table_info('",to or tab,"')"}) do
      table.insert(flds,row.name)
    end

    if #flds == 0 then return end

    flds = table.concat(flds,',')
    local qry = table.concat{'insert or replace into ',app,'.',to or tab,' (',flds,') select ',flds,' from ',from,'.',tab}
    return self.conn:exec(qry)
end

local tbl = {}
function tbl.cmp(one,two,offmt)
   local onetype = type(one)
   local twotype = type(two)
   if onetype ~= twotype then return false end
   if onetype ~= 'table' and twotype ~= 'table' then return one == two end
   local mt = getmetatable(one)
   if not offmt and mt and mt.__eq then return one == two end
   for k,v in pairs(one or {}) do
      local vv = two[k]
      if vv == nil or not tbl.cmp(v,vv) then return false end
   end
   for k,v in pairs(two or {}) do
      local vv = one[k]
      if vv == nil or not tbl.cmp(v,vv) then return false end
   end
   return true
end

function tbl.idx(tb,row)
  for i,line in ipairs(tb or {}) do
    if tbl.cmp(line,row) then
      return i
    end
  end
end

function db.extend(self,mod,ext)
  for key,val in pairs(ext or {}) do
    if not mod[key] then mod[key] = {} end
    debug.log('EXTENDING KEY:',key)
    if key == 'field' and type(val[1]) == 'function' and not mod.field.done then --todo
      val[1](self)
      mod.field.done = true
    end --todo
    if type(val) == 'table' then
      if val.action == 'replace' then
        for i,f in ipairs(mod[key]) do mod[key][i] = nil end
        for k,v in ipairs(val) do
          mod[key][k] = v
        end
        for k,v in pairs(val) do --todo where taxi firm
          if type(k) == 'string' and type(v) == 'table' then
            for kk,vv in pairs(v) do
              if type(kk) == 'string' then
                mod[key][kk] = vv
              elseif type(kk) == 'number' and type(vv) == 'table' then
                if not tbl.idx(mod[key][k],vv) then
                  insert(mod[key][k],vv.insert or #mod[key][k]+1,vv)
                end
              end
            end
          end
        end
      else
        for k,v in pairs(val) do
          debug.log('EXTENDING KEY,VAL,SUPER:',k,v,mod[key][k])
          if type(k) == 'string' then
            mod[key][k] = v
          elseif type(k) == 'number' and type(v) == 'table' then
            if not tbl.idx(mod[key],v) then
              insert(mod[key],v.insert or #mod[key]+1,v)
            end
          end
        end
      end
    elseif type(val) == "function" then
      debug.log('SUPER:',mod.super)
      if not mod.super then
        mod.super = {}
      end
      (mod.super)[key] = mod[key]
      mod[key] = val
    end
    --for k,v in pairs(val) do insert(mod[key],v) end
  end
  return mod
end

return db
