------------------------------
-- yoke © luamani@yahoo.com --
------------------------------

local util = require "yoke.snap.util"
local concat = table.concat

local Request = {}
Request.__index = Request

function Request:new(client, server)
  local req = {}
  req.client = client
  req.server = server
  req.ip = client:getpeername()
  req.line = nil
  req.query = ''
  req.method = nil
  req.path = nil
  req.headers = {}
  return setmetatable(req, self)
end

function Request:getline()
  if self.line then return end

  local status, partial
  self.line, status, partial = self.client:receive()

  if not self.line or status == 'timeout' or status == 'closed' or partial == '' then
    return
  end

  local method, path = string.match(self.line, '^(.-)%s(%S+)%s*(HTTP%/%d%.%d)') -- GET /page HTTP/1.1

  if not method then
    self.client:close()
    self.server:close()
    return
  end

  --print(concat{self.ip,' ',method,' ',path})

  local filename = ''
  local query = ''

  if #path then
    filename, query = string.match(path, '^([^#?]+)[#|?]?(.*)')
    filename = util.url.norm(filename)
  end
  self.path = filename
  self.method = method
  self.query = query
end

function Request:header()
  self:getline()
  local data = self.client:receive()
  while data and data:len() > 0 do
    local key, value = string.match(data, '([%w-]+): ([%w %p]+=?)')
    if key and value then
      --print('HDR:',key,'=',value)
      self.headers[key] = value
    end
    data = self.client:receive()
  end
  return self.headers
end

return Request
